#
# Copyright (c) 2002, Art Haas
#
#
# point tests
#

import unittest
import random

import Generic.point as point

seed = 100
random.seed(seed)

class PointTestCase(unittest.TestCase):
    def setUp(self):
        self.p1 = point.Point(0,0)
        self.p2 = point.Point(10,0)
        self.p3 = point.Point(0,10)
        self.p4 = point.Point(-10,0)
        self.p5 = point.Point(0,-10)

    def testCreation(self):
        self.failUnless(abs(self.p1.x) < 1e-10)
        self.failUnless(abs(self.p1.y) < 1e-10)
        self.failUnless(abs(abs(self.p2.x) - 10) < 1e-10)
        self.failUnless(abs(self.p2.y) < 1e-10)
        self.failUnless(abs(self.p3.x) < 1e-10)
        self.failUnless(abs(abs(self.p3.y) - 10) < 1e-10)
        self.failUnless(abs(abs(self.p4.x) - 10) < 1e-10)
        self.failUnless(abs(self.p4.y) < 1e-10)
        self.failUnless(abs(self.p5.x) < 1e-10)
        self.failUnless(abs(abs(self.p5.y) - 10) < 1e-10)

        self.failUnlessRaises(ValueError, point.Point, (1,2,3))
        self.failUnlessRaises(SyntaxError, point.Point, 1, None)
        self.failUnlessRaises(ValueError, point.Point, 'asdf', 10)
        self.failUnlessRaises(ValueError, point.Point, 10, 'asdf')

    def testDifference(self):
	self.failUnless(abs(self.p1 - self.p1) < 1e-10)
	self.failUnless(abs(abs(self.p1 - self.p2) - 10) < 1e-10)
	self.failUnless(abs(abs(self.p1 - self.p3) - 10) < 1e-10)
	self.failUnless(abs(abs(self.p1 - self.p4) - 10) < 1e-10)
	self.failUnless(abs(abs(self.p1 - self.p5) - 10) < 1e-10)
	self.failUnless(abs(abs(self.p2 - self.p4) - 20) < 1e-10)
	self.failUnless(abs(abs(self.p3 - self.p5) - 20) < 1e-10)

    def testComparisons(self):
        self.failUnless(self.p1 == self.p1)
        self.failUnless(self.p1 < self.p2)
        self.failUnless(self.p1 < self.p3)
        self.failUnless(self.p1 > self.p4)
        self.failUnless(self.p1 > self.p5)
	self.failUnless(self.p2 > self.p1)
        self.failUnless(self.p2 == self.p2)
	self.failUnless(self.p2 > self.p3)
	self.failUnless(self.p2 > self.p4)
	self.failUnless(self.p2 > self.p5)
	self.failUnless(self.p3 > self.p1)
	self.failUnless(self.p3 < self.p2)
        self.failUnless(self.p3 == self.p3)
	self.failUnless(self.p3 > self.p4)
	self.failUnless(self.p3 > self.p5)
	self.failUnless(self.p4 < self.p1)
	self.failUnless(self.p4 < self.p2)
	self.failUnless(self.p4 < self.p3)
        self.failUnless(self.p4 == self.p4)
	self.failUnless(self.p4 < self.p5)
	self.failUnless(self.p5 < self.p1)
	self.failUnless(self.p5 < self.p2)
	self.failUnless(self.p5 < self.p3)
	self.failUnless(self.p5 > self.p4)
        self.failUnless(self.p5 == self.p5)
        
        self.failUnless(self.p1 == (0,0))
        self.failUnless(self.p1 < (10,0))
        self.failUnless(self.p1 < (0,10))
        self.failUnless(self.p1 > (-10,0))
        self.failUnless(self.p1 > (0,-10))
	self.failUnless(self.p2 > (0,0))
        self.failUnless(self.p2 == (10,0))
	self.failUnless(self.p2 > (0,10))
	self.failUnless(self.p2 > (-10,0))
	self.failUnless(self.p2 > (0,-10))
	self.failUnless(self.p3 > (0,0))
	self.failUnless(self.p3 < (10,0))
        self.failUnless(self.p3 == (0,10))
	self.failUnless(self.p3 > (-10,0))
	self.failUnless(self.p3 > (0,-10))
	self.failUnless(self.p4 < (0,0))
	self.failUnless(self.p4 < (10,0))
	self.failUnless(self.p4 < (0,10))
        self.failUnless(self.p4 == (-10,0))
	self.failUnless(self.p4 < (0,-10))
	self.failUnless(self.p5 < (0,0))
	self.failUnless(self.p5 < (10,0))
	self.failUnless(self.p5 < (0,10))
	self.failUnless(self.p5 > (-10,0))
        self.failUnless(self.p5 == (0,-10))

class PointTreeTestCase(unittest.TestCase):
    def setUp(self):
        self.tree = point.PointTree()
        self.p1 = point.Point(0,0)
        self.p2 = point.Point(10,0)
        self.p3 = point.Point(0,10)
        self.p4 = point.Point(-10,0)
        self.p5 = point.Point(0,-10)
        self.p6 = point.Point(0,0)
        self.p7 = point.Point(10,0)
        self.p8 = point.Point(0,10)
        self.p9 = point.Point(-10,0)
        self.p10 = point.Point(0,-10)

    def testStorage(self):
        self.tree.store(self.p1)
        self.failUnless(len(self.tree) == 1)
        self.failUnless(self.p1 in self.tree)
        self.tree.store(self.p2)
        self.failUnless(len(self.tree) == 2)
        self.failUnless(self.p2 in self.tree)
        self.tree.store(self.p3)
        self.failUnless(len(self.tree) == 3)
        self.failUnless(self.p3 in self.tree)
        self.tree.store(self.p4)
        self.failUnless(len(self.tree) == 4)
        self.failUnless(self.p4 in self.tree)
        self.tree.store(self.p5)
        self.failUnless(len(self.tree) == 5)
        self.failUnless(self.p5 in self.tree)
        self.failUnless(self.p6 in self.tree)
        self.failUnless(self.p7 in self.tree)
        self.failUnless(self.p8 in self.tree)
        self.failUnless(self.p9 in self.tree)
        self.failUnless(self.p10 in self.tree)

        for i in xrange(1000):
            x = -100.0 + (random.random() * 200.0)
            y = -100.0 + (random.random() * 200.0)
            p = point.Point(x,y)
            self.tree.store(p)
            
        for i in xrange(1, len(self.tree)):
            pp = self.tree[i-1]
            pi = self.tree[i]
            self.failUnless(pp.x <= pi.x)
            self.failIf(abs(pp.x - pi.x) < 1e-10 and pp.y > pi.y)

    def testBinscan(self):
        self.tree.store(self.p1)
        self.tree.store(self.p2)
        self.tree.store(self.p3)
        self.tree.store(self.p4)
        self.tree.store(self.p5)
        pt = self.tree.binscan(self.p1)
        self.failUnless(pt == self.p1)
        self.failUnless(pt is self.p1)
        pt = self.tree.binscan(self.p2)
        self.failUnless(pt == self.p2)
        self.failUnless(pt is self.p2)
        pt = self.tree.binscan(self.p3)
        self.failUnless(pt == self.p3)
        self.failUnless(pt is self.p3)
        pt = self.tree.binscan(self.p4)
        self.failUnless(pt == self.p4)
        self.failUnless(pt is self.p4)
        pt = self.tree.binscan(self.p5)
        self.failUnless(pt == self.p5)
        self.failUnless(pt is self.p5)
        pt = self.tree.binscan(self.p6)
        self.failUnless(pt == self.p6 and pt is self.p1)
        pt = self.tree.binscan(self.p7)
        self.failUnless(pt == self.p7 and pt is self.p2)
        pt = self.tree.binscan(self.p8)
        self.failUnless(pt == self.p8 and pt is self.p3)
        pt = self.tree.binscan(self.p9)
        self.failUnless(pt == self.p9 and pt is self.p4)
        pt = self.tree.binscan(self.p10)
        self.failUnless(pt == self.p10 and pt is self.p5)

        pt = self.tree.binscan((0,0))
        self.failUnless(pt == (0,0))
        self.failUnless(pt is self.p1)
        self.failUnless(pt == self.p6)
        pt = self.tree.binscan((10,0))
        self.failUnless(pt == (10,0))
        self.failUnless(pt is self.p2)
        self.failUnless(pt == self.p7)
        pt = self.tree.binscan((0,10))
        self.failUnless(pt == (0,10))
        self.failUnless(pt is self.p3)
        self.failUnless(pt == self.p8)
        pt = self.tree.binscan((-10,0))
        self.failUnless(pt == (-10,0))
        self.failUnless(pt is self.p4)
        self.failUnless(pt == self.p9)
        pt = self.tree.binscan((0,-10))
        self.failUnless(pt == (0,-10))
        self.failUnless(pt is self.p5)
        self.failUnless(pt == self.p10)

        rpts = []
        for i in xrange(1000):
            x = -100.0 + (random.random() * 200.0)
            y = -100.0 + (random.random() * 200.0)
            p = point.Point(x,y)
            rpts.append(p)
            self.tree.store(p)
        for i in xrange(len(rpts)):
            rpt = rpts[i]
            pt = self.tree.binscan(rpt)
            self.failUnless(pt is rpt)

    def testScan(self):
        self.tree.store(self.p1)
        self.tree.store(self.p2)
        self.tree.store(self.p3)
        self.tree.store(self.p4)
        self.tree.store(self.p5)
        pt = self.tree.scan(self.p1)
        self.failUnless(pt == self.p1)
        self.failUnless(pt is self.p1)
        pt = self.tree.scan(self.p2)
        self.failUnless(pt == self.p2)
        self.failUnless(pt is self.p2)
        pt = self.tree.scan(self.p3)
        self.failUnless(pt == self.p3)
        self.failUnless(pt is self.p3)
        pt = self.tree.scan(self.p4)
        self.failUnless(pt == self.p4)
        self.failUnless(pt is self.p4)
        pt = self.tree.scan(self.p5)
        self.failUnless(pt == self.p5)
        self.failUnless(pt is self.p5)
        pt = self.tree.scan(self.p6)
        self.failUnless(pt == self.p6 and pt == self.p1)
        pt = self.tree.scan(self.p7)
        self.failUnless(pt == self.p7 and pt == self.p2)
        pt = self.tree.scan(self.p8)
        self.failUnless(pt == self.p8 and pt == self.p3)
        pt = self.tree.scan(self.p9)
        self.failUnless(pt == self.p9 and pt == self.p4)
        pt = self.tree.scan(self.p10)
        self.failUnless(pt == self.p10 and pt == self.p5)

        pt = self.tree.scan((0,0))
        self.failUnless(pt == (0,0))
        self.failUnless(pt is self.p1)
        self.failUnless(pt == self.p6)
        pt = self.tree.scan((10,0))
        self.failUnless(pt == (10,0))
        self.failUnless(pt is self.p2)
        self.failUnless(pt == self.p7)
        pt = self.tree.scan((0,10))
        self.failUnless(pt == (0,10))
        self.failUnless(pt is self.p3)
        self.failUnless(pt == self.p8)
        pt = self.tree.scan((-10,0))
        self.failUnless(pt == (-10,0))
        self.failUnless(pt is self.p4)
        self.failUnless(pt == self.p9)
        pt = self.tree.scan((0,-10))
        self.failUnless(pt == (0,-10))
        self.failUnless(pt is self.p5)
        self.failUnless(pt == self.p10)

        pt = self.tree.scan((0,0.001))
        self.failUnless(pt is None)
        pt = self.tree.scan((0,0.001),0.001)
        self.failUnless(pt == (0,0))
        self.failUnless(pt is self.p1)
        self.failUnless(pt == self.p6)

        pt = self.tree.scan((10,-0.001))
        self.failUnless(pt is None)
        pt = self.tree.scan((10,-0.001),0.001)
        self.failUnless(pt is self.p2)
        self.failUnless(pt == self.p7)

        rpts = []
        for i in xrange(1000):
            x = -100.0 + (random.random() * 200.0)
            y = -100.0 + (random.random() * 200.0)
            p = point.Point(x,y)
            rpts.append(p)
            self.tree.store(p)
        for i in xrange(len(rpts)):
            rpt = rpts[i]
            pt = self.tree.scan(rpt)
            self.failUnless(pt is rpt)
        
    def testRemove(self):
        self.tree.store(self.p1)
        self.tree.store(self.p2)
        self.tree.store(self.p3)
        self.tree.store(self.p4)
        self.tree.store(self.p5)
        self.tree.remove(self.p1)
        self.failUnless(len(self.tree) == 4)
        self.failIf(self.p1 in self.tree)
        self.failIf(self.p6 in self.tree)
        self.tree.remove(self.p2)
        self.failUnless(len(self.tree) == 3)
        self.failIf(self.p2 in self.tree)
        self.failIf(self.p7 in self.tree)
        self.tree.remove(self.p3)
        self.failUnless(len(self.tree) == 2)
        self.failIf(self.p3 in self.tree)
        self.failIf(self.p8 in self.tree)
        self.tree.remove(self.p4)
        self.failUnless(len(self.tree) == 1)
        self.failIf(self.p4 in self.tree)
        self.failIf(self.p9 in self.tree)
        self.tree.remove(self.p5)
        self.failIf(len(self.tree))
        self.failIf(self.p5 in self.tree)
        self.failIf(self.p10 in self.tree)

if __name__ == "__main__":
    PointTestSuite = unittest.TestSuite()
    PointTestSuite.addTest(PointTestCase("testCreation"))
    PointTestSuite.addTest(PointTestCase("testDifference"))
    PointTestSuite.addTest(PointTestCase("testComparisons"))
    
    PointTreeTestSuite = unittest.TestSuite()
    PointTreeTestSuite.addTest(PointTreeTestCase("testStorage"))
    PointTreeTestSuite.addTest(PointTreeTestCase("testBinscan"))
    PointTreeTestSuite.addTest(PointTreeTestCase("testScan"))
    PointTreeTestSuite.addTest(PointTreeTestCase("testRemove"))
    unittest.main()
